/*------------------------------------------------------------------------
    File        : rtb_rpt_class.p
    Purpose     : 

    Syntax      :

    Description : Class-specific Report Library

    Author(s)   : jeff.ledbetter
    Created     :
    Notes       :
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
{rtb/i/rtb_xreftype.i}
{rtb/i/rtb_rptinit.i}

DEFINE TEMP-TABLE ttClass
  FIELD id       AS INTEGER    /* record id */
  FIELD refType  AS INTEGER    /* type of reference */
  FIELD ref      AS CHARACTER  /* reference text */
  FIELD misc     AS CHARACTER  /* extra ref info (desc, params, etc */
  INDEX index1 IS PRIMARY UNIQUE id
  INDEX index2 refType id.

/* ********************  Preprocessor Definitions  ******************** */

/*
 These preprocessors are used as report sections for the Class Details report and as
 a means to sort output.  Changing a number will change the section's order in the
 report output.
*/
&SCOPED-DEFINE SUPERCLASS 1
&SCOPED-DEFINE SUBCLASS 3
&SCOPED-DEFINE INTERFACE 2
&SCOPED-DEFINE METHOD 4
&SCOPED-DEFINE DATAMEMBER 5
&SCOPED-DEFINE PROPERTY 6

/* ***************************  Main Block  *************************** */



/* **********************  Internal Procedures  *********************** */

PROCEDURE add_reference PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:    Add records to the ttClass temp-table
  Parameters: INPUT pcRefType  - the type of reference (see preprocessors)
              INPUT pcRefText  - referenced object
              INPUT pcMisc     - extra info about the object
  Notes:      
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pcRefType  AS INTEGER  NO-UNDO.
  DEFINE INPUT PARAMETER pcRefText  AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pcMisc     AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iRefCnt AS INTEGER NO-UNDO.

  DEFINE BUFFER bttClass FOR ttClass.

  FIND LAST bttClass NO-ERROR.
  IF AVAILABLE bttClass THEN 
    iRefCnt = bttClass.id + 1.
  ELSE
    iRefCnt = 1.

  CREATE ttClass.
  ASSIGN
    ttClass.id       = iRefCnt
    ttClass.refType  = pcRefType
    ttClass.ref      = pcRefText
    ttClass.misc     = pcMisc.

  RETURN.

END PROCEDURE.

PROCEDURE find_other_references PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:     Find other references we might be interested in.
  Parameters:  INPUT prRecid - RECID of the rtb_object record on which we are
                               reporting.
  Notes:       This looks at 'xref data' for the given class for interface
               implementations, data-members, and methods
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER prRecid AS RECID     NO-UNDO.

  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.
  DEFINE BUFFER B2rtb_object FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_xref    FOR rtb.rtb_xref.

  FIND Brtb_object NO-LOCK
    WHERE RECID(Brtb_object) = prRecid.

  FOR EACH Brtb_xref NO-LOCK
    WHERE Brtb_xref.src-recid = RECID(Brtb_object):

    /*
     Interface implementations.. 
    */
    IF Brtb_xref.ref-type = {&XREF_IMPLEMENTATION} THEN DO:

      FIND B2rtb_object NO-LOCK
        WHERE RECID(B2rtb_object) = Brtb_xref.ref-recid1.

      FIND Brtb_ver NO-LOCK
        OF B2rtb_object.

      RUN add_reference
        (INPUT {&INTERFACE},
         INPUT Brtb_xref.ref-text,
         Brtb_ver.DESCRIPTION).
  
    END.

    /*
     Defined methods..
     We are filtering on action because we don't want any invoked methods.
    */
    IF Brtb_xref.ref-type = {&XREF_METHOD} AND Brtb_xref.ref-actions = "" THEN
      RUN add_reference
        (INPUT {&METHOD},
         INPUT Brtb_xref.obj-text,
         "").  /* maybe one day pass the method parameters to the report? */

    /*
     Data members.. 
    */
    IF Brtb_xref.ref-type = {&XREF_DATAMEMBER} THEN
      RUN add_reference
        (INPUT {&DATAMEMBER},
         INPUT Brtb_xref.obj-text,
         ""). /* maybe one day pass datamember data to the report? */

    /*
     Properties.. 
    */
    IF Brtb_xref.ref-type = {&XREF_PROPERTY} THEN
      RUN add_reference
        (INPUT {&PROPERTY},
         INPUT Brtb_xref.obj-text,
         "").

  END.

  RETURN.

END PROCEDURE.

PROCEDURE find_subclasses PRIVATE:
/*------------------------------------------------------------------------------
  Purpose:     Find any subclasses of the given class.
  Parameters:  INPUT prRecid - RECID of the rtb_object record on which we are
                               reporting.
  Notes:       This looks at 'where-used' data to find if any other class has
               subclassed this one.
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER prRecid AS RECID NO-UNDO.

  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.
  DEFINE BUFFER B2rtb_object FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_xref    FOR rtb.rtb_xref.

  FIND Brtb_object NO-LOCK
    WHERE RECID(Brtb_object) = prRecid.

  FOR EACH Brtb_xref NO-LOCK
    WHERE Brtb_xref.ref-recid1 = RECID(Brtb_object)
      AND Brtb_xref.src-type   = {&XREF_CLASS}:

    IF Brtb_xref.ref-type = {&XREF_SUBCLASS} THEN DO:

      FIND B2rtb_object NO-LOCK
        WHERE RECID(B2rtb_object) = Brtb_xref.src-recid.

      FIND Brtb_ver NO-LOCK
        OF B2rtb_object.

      RUN add_reference
        (INPUT {&SUBCLASS},
         INPUT B2rtb_object.OBJECT,
         INPUT Brtb_ver.DESCRIPTION).

    END.

  END.

  RETURN.
END PROCEDURE.

PROCEDURE find_superclasses PRIVATE:
/*------------------------------------------------------------------------------
  Purpose:     Look for superClass records in the class hierarchy for the given
               class and add them to our temp-table.
  Parameters:  INPUT prRecid - RECID of rtb_object record for the class
  Notes:       We do a little recursion here to build the list of superClasses
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER prRecid AS RECID     NO-UNDO.

  DEFINE BUFFER Brtb_object FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_xref   FOR rtb.rtb_xref.

  FIND Brtb_object NO-LOCK
    WHERE RECID(Brtb_object) = prRecid.

  RUN add_reference
    (INPUT {&SUPERCLASS},
     INPUT Brtb_object.OBJECT,
     "").

  /*
   Find superClass of class
  */
  FIND FIRST Brtb_xref NO-LOCK
    WHERE Brtb_xref.src-recid = RECID(Brtb_object)
      AND Brtb_xref.ref-type  = {&XREF_SUBCLASS} NO-ERROR.

  /*
   If ref-recid1 is not null, then the superclass is an object known to
   Roundtable. Otherwise, just simply add the obj-text which will contain
   the name of the parent class.
  */
  IF AVAILABLE Brtb_xref AND Brtb_xref.ref-recid1 <> ? THEN
    RUN find_superclasses (Brtb_xref.ref-recid1). /* ooh.. ah.. recursion */
  ELSE IF Brtb_xref.obj-text <> "" THEN
    RUN add_reference ({&SUPERCLASS}, Brtb_xref.obj-text, "").

  RETURN.

END PROCEDURE.

PROCEDURE rtb_class_details PRIVATE:
/*------------------------------------------------------------------------------
  Purpose:    Main procedure for the Class Details report
  Parameters: INPUT Pwspace  - workspace id
              INPUT PobjType - object type
              INPUT Pobject  - class to report on
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace  AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER PobjType AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER Pobject  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE i AS INTEGER NO-UNDO.

  DEFINE BUFFER Brtb_object FOR rtb.rtb_object.

  FIND Brtb_object NO-LOCK
    WHERE Brtb_object.wspace-id = Pwspace
      AND Brtb_object.obj-type  = PobjType
      AND Brtb_object.OBJECT    = Pobject NO-ERROR.

  IF NOT AVAILABLE Brtb_object THEN
    RETURN "Object record is not available.".

  /*
   Gather report data..
  */
  RUN find_superclasses (RECID(Brtb_object)).
  RUN find_subclasses (RECID(Brtb_object)).  
  RUN find_other_references (RECID(Brtb_object)).

  /*
   Print the report.. 
  */
  {rtb/i/rtb_rpthdr.i 
     &title="'Class Details'"
     &criteria="'Class: ' Brtb_object.object"
     &center-criteria=NO
     &STREAM=Sout}

  /*
   superClass hiearchy..
  */
  PUT STREAM sOut "Class Hierarchy:" SKIP(1).

  FOR EACH ttClass
    WHERE ttClass.refType = {&SUPERCLASS}
    BY ttClass.id DESCENDING:

    i = i + 1.

    PUT STREAM sOut UNFORMATTED FILL(" ", i * 4 - 2). /* this faaancy calculation indents two spaces for each branch */

    IF i GT 1 THEN
        PUT STREAM sOut UNFORMATTED "|_". /* we only want this on secondary levels */

    PUT STREAM sOut UNFORMATTED ttClass.ref.

    IF ttClass.ref = Brtb_object.object THEN
      PUT STREAM sOut UNFORMATTED " *".

    PUT STREAM sOut SKIP.

  END.

  /*
   Process the rest of the temp-table breaking at each section.
  */
  FOR EACH ttClass
    BREAK BY ttClass.refType:

    IF FIRST-OF(ttClass.refType) THEN DO:

      CASE ttClass.refType:

      WHEN {&INTERFACE} THEN
        PUT STREAM sOut SKIP(2) "All Implemented Interfaces:" SKIP.

      WHEN {&SUBCLASS} THEN
        PUT STREAM sOut SKIP(2) "Direct Known Subclasses:" SKIP.

      WHEN {&METHOD} THEN
        PUT STREAM sOut SKIP(2) "Method Summary:" SKIP.

      WHEN {&DATAMEMBER} THEN
        PUT STREAM sOut SKIP(2) "Data Members:" SKIP.

      WHEN {&PROPERTY} THEN
        PUT STREAM sOut SKIP(2) "Properties:" SKIP.

      END CASE.

    END. /* first-of */

    CASE ttClass.refType:

    WHEN {&INTERFACE} THEN
      DISPLAY STREAM sOut
        SPACE(2)
        ttClass.ref  FORMAT "x(40)" LABEL "Interface"
        ttClass.misc FORMAT "x(40)" LABEL "Description"
        WITH FRAME frInterface WIDTH 120 DOWN STREAM-IO.

    WHEN {&SUBCLASS} THEN
      DISPLAY STREAM sOut
        SPACE(2)
        ttClass.ref  FORMAT "x(40)" LABEL "Subclass"
        ttClass.misc FORMAT "x(40)" LABEL "Description"
        WITH FRAME frSubclass WIDTH 120 DOWN STREAM-IO.

    WHEN {&METHOD} THEN 
      DISPLAY STREAM sOut
        SPACE(2)
        ttClass.ref  FORMAT "x(40)" LABEL "Method"
        WITH FRAME frMethod WIDTH 120 DOWN STREAM-IO.

    WHEN {&DATAMEMBER} THEN
      DISPLAY STREAM sOut
        SPACE(2)
        ttClass.ref  FORMAT "x(40)" LABEL "Data Member"
        WITH FRAME frDatamember WIDTH 120 DOWN STREAM-IO.

    WHEN {&PROPERTY} THEN
      DISPLAY STREAM sOut
        SPACE(2)
        ttClass.ref  FORMAT "x(40)" LABEL "Property"
        WITH FRAME frProperty WIDTH 120 DOWN STREAM-IO.

    END CASE.

  END. /* each ttClass */

  {rtb/i/rtb_rptftr.i &STREAM=Sout}

  EMPTY TEMP-TABLE ttClass.
  
  RETURN.

END PROCEDURE.

PROCEDURE rptRtbClassDetails :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace  AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER PobjType AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER Pobject  AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_class_details (INPUT Pwspace, INPUT PobjType, INPUT Pobject).

  OUTPUT STREAM Sout CLOSE.

  RETURN RETURN-VALUE.

END PROCEDURE.
